import tempfile
import fitz  # PyMuPDF
from PIL import Image
import easyocr
import os
import gc
def pdf_to_images(pdf_path, dpi=300):
    # Open the PDF file
    pdf_document = fitz.open(pdf_path)
    images_paths = []
    temp = tempfile.mkdtemp(prefix="pre_",suffix="_suf")
    # Loop through all pages
    for page_number in range(len(pdf_document)):
        # Select the page
        page = pdf_document.load_page(page_number)
        # Convert page to a pixmap (image)
        zoom = dpi / 72  # DPI conversion factor
        mat = fitz.Matrix(zoom, zoom)
        pix = page.get_pixmap(matrix=mat)
        # Save the pixmap as an image
        img_path = f'{temp}\\page_{page_number+1}.png'
        pix.save(img_path)
        images_paths.append(img_path)
        #os.remove(image_path)
        del img_path
        del pix
    return images_paths

def crop_image(image_path, crop_box):
    # Open the image
    temp = tempfile.mkdtemp(prefix="pre_",suffix="_suf")
    with Image.open(image_path) as img:
        # Crop the image
        cropped_img = img.crop(crop_box)
        # Save the cropped image
        cropped_img_path = f'{temp}\\page.png'
        cropped_img.save(cropped_img_path, dpi=(img.info['dpi']))
        #os.remove(cropped_img_path)
        del cropped_img
    return cropped_img_path

# Example usagepy
class MultiPdf :
    def MultiOcr(pdf_path,crop_box,dpi=300):
        AllResult = []
        gc.enable()
        gc.set_threshold(600, 50, 50)
        reader = easyocr.Reader(['en'], gpu=True)
        images_paths = pdf_to_images(pdf_path, dpi)
        folder_path = os.path.dirname(images_paths[0])
        for image_path in images_paths:
            cropped_image_path = crop_image(image_path, crop_box)
            allResults = reader.readtext(cropped_image_path)
            AllResult.append(allResults)
            folder_image_path = os.path.dirname(cropped_image_path)
            os.remove(cropped_image_path)
            os.remove(image_path)
            del image_path
            del allResults
            del cropped_image_path
            if not os.listdir(folder_image_path):
                os.rmdir(folder_image_path)
        if not os.listdir(folder_path):
            os.rmdir(folder_path)
        gc.collect()
        gc.disable()
        return AllResult


import easyocr
import cv2
from pyzbar.pyzbar import decode
import os
import tempfile
from ironpdf import PdfDocument
import PyPDF2

class OcrTools:
    def __init__(self, pdfPath):
        self.pdfPath = pdfPath

    def pdfToImg(pdfPath="", locationFile=""):
        pdf = PdfDocument.FromFile(pdfPath)
        # Extract all pages to a folder as image files
        pdf.RasterizeToImageFiles(locationFile, DPI=300)
        return locationFile

    def count_pdf_pages(file_path):
        file = open(file_path,'rb')
        pdfReader = PyPDF2.PdfReader(file)
        totalPages = len(pdfReader.pages)
        print(f"Total Pages: {totalPages}")
        return totalPages
    
    def easyOCR(typeOfOcr='Text', locationFile="",
                area={'offset_x_min': 0, 'offset_y_min': 0,
                      'width': 0, 'height': 0},
                dataCheck=['valueForCheck'],
                languageOcr=['th', 'en'],
                deviation=50
                ):
        temp = tempfile.mkdtemp(prefix="pre_",suffix="_suf")
        name, extension = os.path.splitext(locationFile)
        pdfCount=OcrTools.count_pdf_pages(locationFile)
        if extension == ".pdf":
            root = locationFile.find("/")
            if root > 0:
                newFileLocation = locationFile.rsplit('/', 1)[1]
            else:
                newFileLocation = "."
            print(newFileLocation)
            imageLocation = OcrTools.pdfToImg(
                locationFile, temp+"\\"+newFileLocation+".png")
        else:
            imageLocation = locationFile
        area_xMax = area['offset_x_min']+area['width']
        area_yMax = area['offset_y_min']+area['height']
        allResultDetective = []
        for i in range(pdfCount):
            if pdfCount >1:
                i+=1
                print(i,imageLocation)
                fileLocation = imageLocation.replace(".pdf", ".pdf_pg"+str(i))
            else:
                fileLocation = imageLocation
            if typeOfOcr == 'Text':
                reader = easyocr.Reader(languageOcr)
                allResults = reader.readtext(fileLocation)
                for result in allResults:
                    print(result)
                    pos1, pos2, pos3, pos4 = result[0]
                    xMin = round(min([pos1[0], pos2[0], pos3[0], pos4[0]]))
                    xMax = round(max([pos1[0], pos2[0], pos3[0], pos4[0]]))
                    yMin = round(min([pos1[1], pos2[1], pos3[1], pos4[1]]))
                    yMax = round(max([pos1[1], pos2[1], pos3[1], pos4[1]]))
                    if (area['offset_x_min'] in range(xMin-deviation, xMin+deviation)):
                        if (area['offset_y_min'] in range(yMin-deviation, yMin+deviation)):
                            if (area_xMax in range(xMax-deviation, xMax+deviation)):
                                if (area_yMax in range(area_yMax-deviation, area_yMax+deviation)):
                                    word = result[1]
                                    print(word)
                                    for wordCheck in dataCheck:
                                        if word.lower() == wordCheck.lower():
                                            valid = True
                                        else:
                                            valid = False
                                    accuracy_percent = str(
                                        round(result[2]*100))+"%"
                                    allResultDetective.append({
                                        'page':i,
                                        'coordinates': result[0],
                                        'value': word,
                                        'valid': valid,
                                        'accuracy_percent': accuracy_percent,
                                    })
                os.remove(fileLocation)
            elif typeOfOcr == 'qrcode':
                image = cv2.imread(fileLocation)
                cropped_image = image[area['offset_y_min']:area_yMax, area['offset_x_min']:area_xMax]
                result = decode(cropped_image)
                os.remove(fileLocation)
                if result:
                    allResultDetective.append({
                        'page':i,
                        'coordinates': [area['offset_y_min'], area_yMax, area['offset_x_min'], area_xMax],
                        'value': result[0].data,
                        'valid': True,
                        'accuracy_percent': '-',
                    }) 
                else:
                    allResultDetective.append({
                        'page':i,
                        'coordinates': [area['offset_y_min'], area_yMax, area['offset_x_min'], area_xMax],
                        'value': result,
                        'valid': False,
                        'accuracy_percent': '-',
                    })
        return allResultDetective



import io
from os.path import abspath, dirname, join
from setuptools import find_packages, setup


HERE = dirname(abspath(__file__))
LOAD_TEXT = lambda name: io.open(join(HERE, name), encoding='UTF-8').read()
DESCRIPTION = '\n\n'.join(LOAD_TEXT(_) for _ in [
    'README.rst'
])

setup(
  name = 'OCR_GLS_G6',      
  packages = ['OCR_GLS_G6'], 
  version = '1.0',  
  license='MIT', 
  description = 'OCR_GLS_G6 - Optical character recognition and QR codes. Support Multiple PDF page file.',
  long_description=DESCRIPTION,
  author = 'Burin Panchat',                 
  author_email = 'burin.gbp@gmail.com',     
  url = 'https://git.bdms.co.th/Burin.Pa/ocr_gls_g6',  
  download_url = 'https://git.bdms.co.th/Burin.Pa/ocr_gls_g6/-/blob/master/dist/ocr_gls_g6.zip',  
  keywords = ['OCR_GLS_G6', 'OCR', 'ocr'],   
  classifiers=[
    'Development Status :: 3 - Alpha',     
    'Intended Audience :: Developers',     
    'Topic :: Documentation',
    'License :: OSI Approved :: MIT License',        
    'Programming Language :: Python :: 3.7',
  ],
  install_requires=[
    "easyocr==1.7.1",
    "pyzbar==0.1.9",
    "IronPdf==2024.5.1.2",
    "PyPDF2==3.0.1",
    "PyMuPDF",
    "pillow",
    ],
)